@@ -8,7 +8,7 @@ use warnings;
 use parent qw{ Specio::Exporter };
 
 use Carp;
-use Date::Tolkien::Shire::Data qw{
+use Date::Tolkien::Shire::Data 0.001 qw{
     __holiday_name_to_number
     __month_name_to_number
 };
@@ -18,7 +18,7 @@ use Specio::Library::Builtins;
 use Specio::Library::Numeric;
 use Specio::Library::String;
 
-our $VERSION = '0.900';
+our $VERSION = '0.901';
 
 declare(
     DayOfMonth	=>
@@ -36,6 +36,45 @@ declare(
     },
 );
 
+=begin comment
+
+object_isa_type(
+    Duration	=>
+    class	=> 'DateTime::Fiction::JRRTolkien::Shire::Duration',
+#    class	=> 'DateTime::Duration',
+);
+
+=end comment
+
+=cut
+
+# For some reason the above does not compile, though if you specify
+# 'DateTime::Duration' it does. I believe the following to be
+# equivalent, though I am sure it will produce a less-salubrious error
+# message.
+
+declare(
+    Duration	=>
+    parent	=> t( 'Object' ),
+#    inline	=> sub {
+#	$_[0]->parent->inline_check( $_[1] ) .
+#	" && $_[1]->isa( 'DateTime::Fiction::JRRTolkien::Shire::Duration' )";
+#    },
+#   For some reason the above is a problem under 5.8.9. So:
+    where	=> sub { $_[0]->isa(
+	    'DateTime::Fiction::JRRTolkien::Shire::Duration' ) },
+);
+
+enum(
+    EndOfMonthMode	=>
+    values	=> [ qw{ limit preserve wrap } ],
+);
+
+any_can_type(
+    Formatter	=>
+    methods	=> [ qw{ format_datetime } ],
+);
+
 declare(
     HolidayName	=>
     parent	=> t( 'NonEmptySimpleStr' ),
@@ -69,12 +108,16 @@ declare(
     },
 );
 
+union(
+    IntOrUndef	=>
+    of	=> [ map { t( $_ ) } qw{ Int Undef } ],
+);
+
 
 declare(
-    'LocaleObject',
-    parent => t('Object'),
-    inline => sub {
-
+    LocaleObject	=>
+    parent	=> t( 'Object' ),
+    inline	=> sub {
         # Can't use $_[1] directly because 5.8 gives very weird errors
         my $var = $_[1];
         <<"EOF";
@@ -87,7 +130,7 @@ EOF
 );
 
 union(
-    'Locale',
+    Locale	=>
     of => [ map { t( $_ ) } qw{ NonEmptySimpleStr LocaleObject } ],
 );
 
@@ -177,11 +220,6 @@ declare(
     parent	=> t( 'Int' ),
 );
 
-any_can_type(
-    'Formatter',
-    methods => ['format_datetime'],
-);
-
 1;
 
 __END__
